#include <iostream>

#include <qcstring.h>
#include <qdatastream.h>
#include <qfile.h>
#include <qimage.h>
#include <qtextstream.h>

QImage * setAlpha (const QImage & img, int alpha)
{
		int w=img.width();
		int h=img.height();

		QImage * dst_img = new QImage (w, h, 32);
		dst_img->setAlphaBuffer (true);

		for (int y=0; y<h; ++y) {
			uint * src_line = (uint*) img.scanLine(y);
			uint * dst_line = (uint*) dst_img->scanLine(y);
			for (int x=0; x<w; ++x) {
				int r = qRed(*(src_line+x));
				int g = qGreen(*(src_line+x));
				int b = qBlue(*(src_line+x));
				*(dst_line) = qRgba (r, g, b, alpha);
			}
		}

		return dst_img;
}

QImage * darkenImage (const QImage & img, int factor=200)
{
		int w=img.width();
		int h=img.height();

		QImage * dst_img = new QImage (w, h, 32);
		dst_img->setAlphaBuffer (true);

		for (int y=0; y<h; ++y) {
			uint * src_line = (uint*) img.scanLine(y);
			uint * dst_line = (uint*) dst_img->scanLine(y);
			for (int x=0; x<w; ++x) {
				int alpha = qAlpha (*(src_line+x));
				QColor c = QColor(*(src_line+x)).dark(factor);
				*(dst_line+x) = qRgba (c.red(), c.green(), c.blue(), alpha);
			}
		}

		return dst_img;
}

QImage * createAlphaMask (const QImage & img)
{
	int w=img.width();
	int h=img.height();

	QImage * dst_img = new QImage (w, h, 32);
	dst_img->setAlphaBuffer (true);
	
	for (int y=0; y<h; ++y) {
			uint * src_line = (uint*) img.scanLine(y);
			uint * dst_line = (uint*) dst_img->scanLine(y);
			for (int x=0; x<w; ++x) {
				int gray = qGray (*(src_line+x));
				int value = (int) (255 - 2*QABS(gray-127.5));
				*(dst_line+x) = qRgb(value, value, value);
			}
	}

	return dst_img;
}

void printUsage ()
{
		std::cerr <<
				"Usage: convert --alpha <alpha> <src> <dst>" << std::endl <<
				"       convert --alpha-mask <src> <dst>" << std::endl <<
				"       convert --darken <percent> <src> <dst>" << std::endl;
}

int main(int argc, char ** argv)
{
	if (argc < 2) {
			printUsage();
			exit(1);
	}
		
	QString alpha_arg = "--alpha";
	QString alpha_mask_arg = "--alpha-mask";
	QString darken_arg = "--darken";
	
	if (alpha_arg == argv[1] || darken_arg == argv[1])
	{
		if (argc != 5) {
				printUsage();
				exit(1);
		}
		QImage src (argv[3]);
		if (src.isNull()) {
			std::cerr << "Source image not found." << std::endl;
			exit (1);
		}
		QImage * dst;
		if (alpha_arg == argv[1]) {
			QString alpha_string = argv[2];
			dst = setAlpha (src, alpha_string.toInt());
		}
		else {
			QString darken_string = argv[2];
			dst = darkenImage (src, darken_string.toInt());
		}
		if (! dst->save (argv[4], "PNG")) {
			std::cerr << "Destination image could not be written." << std::endl;
			exit (1);
		}
	}
	else if (alpha_mask_arg == argv[1])
	{
			if (argc != 4) {
				printUsage();
				exit (1);
			}
			QImage src (argv[2]);
			if (src.isNull()) {
				std::cerr << "Source image not found." << std::endl;
				exit (1);
			}
			QImage * dst = createAlphaMask (src);
			if (! dst->save (argv[3], "PNG")) {
				std::cerr << "Destination image could not be written." <<
						std::endl;
				exit (1);
			}
	}
	else
	{
			printUsage();
			exit (1);
	}
}

